'==============================================================================
'
' Array Manipulation Routines for Visual Basic Programmers
' Copyright (c) 1996,97 by PowerBASIC, Inc. All Rights Reserved.
'
'------------------------------------------------------------------------------
'
' Routines for VB3 and VB4/16 users to manipulate arrays.  These routines
' only support single dimensional arrays.
'
' Requires the PowerBASIC DLL Compiler for Windows (PB/DLL) version 1.5
' or later to recompile.
'
' To use these routines, copy the declare statements below, into the
' Declarations section of your form or module.  Then call them as you would
' any VB sub or function.
'
'   Declare Sub SortString Lib "VBARRAY.DLL" (x$())
'   Declare Sub SortInteger Lib "VBARRAY.DLL" (x%())
'   Declare Sub SortLong Lib "VBARRAY.DLL" (x&())
'   Declare Sub SortSingle Lib "VBARRAY.DLL" (x!())
'   Declare Sub SortCurrency Lib "VBARRAY.DLL" (x@())
'
'   Declare Function SumInteger! Lib "VBARRAY.DLL" (x%())
'   Declare Function SumLong! Lib "VBARRAY.DLL" (x&())
'   Declare Function SumSingle! Lib "VBARRAY.DLL" (x!())
'
'==============================================================================

$COMPILE DLL "VBARRAY.DLL"   'compile to a DLL
$DIM ALL                     'require all variables to be declared first
$HUGE                        'support arrays larger than 64k

$INCLUDE "WINAPI.INC"        'requires the Windows API
$INCLUDE "VBAPI.INC"         ' and Visual Basic API

'==============================================================================

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SortString - Sort a Visual Basic string array.
'
SUB SortString(BYVAL hArray AS DWORD) EXPORT

  LOCAL Hlstr    AS DWORD
  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Buffer   AS STRING
  LOCAL Temp     AS STRING

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Create a PowerBASIC array with the same dimensions
  DIM pbArray(VbLBound TO VbUBound) AS STRING

  ' Fill the PowerBASIC array
  FOR x = VbLBound TO VbUBound
    Hlstr  = VBArrayElement(hArray, 1, x)
    pbArray(x) = VbStringToPb(HlStr)
  NEXT x

  ' Sort the array
  ARRAY SORT pbArray()

  ' Copy the sorted data back to the Visual Basic array
  FOR x = VbLBound TO VbUBound
    Hlstr  = VBArrayElement(hArray, 1, x)
    SetVbString Hlstr, pbArray(x)
  NEXT x

END SUB


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SortInteger - Sort a Visual Basic integer array
'
SUB SortInteger(BYVAL hArray AS DWORD) EXPORT

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS INTEGER PTR

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Create a PowerBASIC array with the same dimensions
  DIM pbArray(VbLBound TO VbUBound) AS INTEGER

  ' Fill the PowerBASIC array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    pbArray(x) = @Var
  NEXT x

  ' Sort the array
  ARRAY SORT pbArray()

  ' Copy the sorted data back to the Visual Basic array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    @Var = pbArray(x)
  NEXT x

END SUB


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SortLong - Sort a Visual Basic long integer array
'
SUB SortLong(BYVAL hArray AS DWORD) EXPORT

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS LONG PTR

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Create a PowerBASIC array with the same dimensions
  DIM pbArray(VbLBound TO VbUBound) AS LONG

  ' Fill the PowerBASIC array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    pbArray(x) = @Var
  NEXT x

  ' Sort the array
  ARRAY SORT pbArray()

  ' Copy the sorted data back to the Visual Basic array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    @Var = pbArray(x)
  NEXT x

END SUB


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SortSingle - Sort a Visual Basic single-precision array
'
SUB SortSingle(BYVAL hArray AS DWORD) EXPORT

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS SINGLE PTR

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Create a PowerBASIC array with the same dimensions
  DIM pbArray(VbLBound TO VbUBound) AS SINGLE

  ' Fill the PowerBASIC array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    pbArray(x) = @Var
  NEXT x

  ' Sort the array
  ARRAY SORT pbArray()

  ' Copy the sorted data back to the Visual Basic array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    @Var = pbArray(x)
  NEXT x

END SUB


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SortCurrency - Sort a Visual Basic currency array
'
SUB SortCurrency(BYVAL hArray AS DWORD) EXPORT

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS CURRENCY PTR

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Create a PowerBASIC array with the same dimensions
  DIM pbArray(VbLBound TO VbUBound) AS CURRENCY

  ' Fill the PowerBASIC array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    pbArray(x) = @Var
  NEXT x

  ' Sort the array
  ARRAY SORT pbArray()

  ' Copy the sorted data back to the Visual Basic array
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    @Var = pbArray(x)
  NEXT x

END SUB


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SumInteger - Return the sum of all values in an integer array
'
FUNCTION SumInteger(BYVAL hArray AS DWORD) EXPORT RETPRM AS SINGLE

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS INTEGER PTR
  LOCAL Sum      AS SINGLE

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Add up all the array elements
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    Sum = Sum + @Var
  NEXT x

  ' Return the result
  FUNCTION = Sum

END FUNCTION


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SumLong - Return the sum of all values in a long integer array
'
FUNCTION SumLong(BYVAL hArray AS DWORD) EXPORT RETPRM AS SINGLE

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS LONG PTR
  LOCAL Sum      AS SINGLE

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Add up all the array elements
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    Sum = Sum + @Var
  NEXT x

  ' Return the result
  FUNCTION = Sum

END FUNCTION


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SumSingle - Return the sum of all values in a single-precision array
'
FUNCTION SumSingle(BYVAL hArray AS DWORD) EXPORT RETPRM AS SINGLE

  LOCAL VbLBound AS LONG
  LOCAL VbUBound AS LONG
  LOCAL x        AS INTEGER
  LOCAL Var      AS SINGLE PTR
  LOCAL Sum      AS SINGLE

  ' Get lower bound of first index
  vbLBound = CLNG(VBArrayBounds(hArray, 1) AND &HFFFF)

  ' Get upper bound of first index
  VbUBound = CLNG(VBArrayBounds(hArray, 1) \ &H10000)

  ' Add up all the array elements
  FOR x = VbLBound TO VbUBound
    Var  = VBArrayElement(hArray, 1, x)
    Sum = Sum + @Var
  NEXT x

  ' Return the result
  FUNCTION = Sum

END FUNCTION

