'============================================================================
'
'  Math DLL example for PB/DLL
'  Copyright (c) 1996,97 by PowerBASIC, Inc.  All Rights Reserved.
'
'----------------------------------------------------------------------------
'                            Conversion Functions
'----------------------------------------------------------------------------
'  CelsiusToFahr      - Celsius to Fahrenheit
'  CentToInch         - Centimeters to Inches
'  DegreesToRadians   - Degrees to Radians
'  Factorial          - Factorial
'  FahrToCelsius      - Fahrenheit to Celsius
'  GallonsToLiters    - Gallons to Liters
'  InchToCent         - Inches to Centimeters
'  KilogramsToPounds  - Kilograms to Pounds
'  KilometersToMiles  - Kilometers to Miles
'  LitersToGallons    - Liters to Gallons
'  MilesToKilometers  - Miles to Kilometers
'  PoundsToKilograms  - Pounds to Kilograms
'  RadiansToDegrees   - Radians to Degrees
'----------------------------------------------------------------------------
'                          Derived math functions
'----------------------------------------------------------------------------
'  Secant             - Secant
'  Cosecant           - Cosecant
'  Cotangent          - Cotangent
'
'  ArcSin             - Inverse Sine
'  ArcCos             - Inverse Cosine
'  ArcSec             - Inverse Secant
'  ArcCosec           - Inverse Cosecant
'  ArcCotan           - Inverse Cotangent
'
'  SinH               - Hyperbolic Sine
'  CosH               - Hyperbolic Cosine
'  TanH               - Hyperbolic Tangent
'  SecH               - Hyperbolic Secant
'  CosecH             - Hyperbolic Cosecant
'  CotanH             - Hyperbolic Cotangent
'
'  ArcSineH           - Inverse Hyperbolic Sine
'  ArcCosH            - Inverse Hyperbolic Cosine
'  ArcTanH            - Inverse Hyperbolic Tangent
'  ArcSecH            - Inverse Hyperbolic Secant
'  ArcCosecH          - Inverse Hyperbolic Cosecant
'  ArcCotanH          - Inverse Hyperbolic Cotangent
'
'  LogN               - Logarithm
'----------------------------------------------------------------------------

$COMPILE  DLL       ' Compile to a DLL

DEFSNG    A-Z       ' default to single precision

'==============================================================================
'                            Conversion functions
'==============================================================================


'------------------------------------------------------------------------------
' Factorial - Calculate the factorial for a given value
'
FUNCTION Factorial(BYVAL Value AS INTEGER) EXPORT AS INTEGER

  DIM Temp AS INTEGER
  DIM x    AS INTEGER

  IF (Value = 1) OR (Value = 0) THEN
    FUNCTION = 1
    EXIT FUNCTION
  ELSEIF Value < 0 OR Value > 1754 THEN
    FUNCTION = -1
    EXIT FUNCTION
  END IF

  Temp = 1
  FOR x% = 2 TO Value
    Temp = Temp * x%
  NEXT x%

  FUNCTION = Temp

END FUNCTION


'------------------------------------------------------------------------------
' FahrToCelsius - Convert degrees fahrenheit to degrees celsius
'
FUNCTION FahrToCelsius(BYVAL DegF) RETPRM EXPORT

   FUNCTION = ( DegF - 32 ) / 1.8

END FUNCTION


'------------------------------------------------------------------------------
' CelsiusToFahr - Convert degrees celsius to degrees fahrenheit
'
FUNCTION CelsiusToFahr(BYVAL DegC) RETPRM EXPORT

   FUNCTION = DegC * 1.8 + 32

END FUNCTION


'------------------------------------------------------------------------------
' KilogramsToPounds - Convert kilograms to pounds
'
FUNCTION KilogramsToPounds(BYVAL Kg) RETPRM EXPORT

  FUNCTION = Kg * 2.2046223

END FUNCTION


'------------------------------------------------------------------------------
' PoundsToKilograms - Convert pounds to kilograms
'
FUNCTION PoundsToKilograms(BYVAL Lbs) RETPRM EXPORT

  FUNCTION = Lbs / 2.2046223

END FUNCTION


'------------------------------------------------------------------------------
' CentToInch - Convert centimeters into inches
'
FUNCTION CentToInch(BYVAL Cent) RETPRM EXPORT

  FUNCTION = Cent * 0.3937

END FUNCTION


'------------------------------------------------------------------------------
' InchToCent - Convert inches into centimeters
'
FUNCTION InchToCent(BYVAL Inches) RETPRM EXPORT

  FUNCTION = Inches * 2.54

END FUNCTION


'------------------------------------------------------------------------------
' MilesToKilometers - Convert miles into kilometers
'
FUNCTION MilesToKilometers(BYVAL Miles) RETPRM EXPORT

  FUNCTION = Miles * 1.609344

END FUNCTION


'------------------------------------------------------------------------------
' KilometersToMiles - Convert kilometers into miles
'
FUNCTION KilometersToMiles(BYVAL Kilometers) RETPRM EXPORT

  FUNCTION = Kilometers * .6213712

END FUNCTION


'------------------------------------------------------------------------------
' DegreesToRadians - Convert degrees to radians
'
FUNCTION DegreesToRadians(BYVAL Degrees) RETPRM EXPORT

  FUNCTION = Degrees * 0.01745

END FUNCTION


'------------------------------------------------------------------------------
' RadiansToDegrees - Convert radians to degrees
'
FUNCTION RadiansToDegrees(BYVAL Radians) RETPRM EXPORT

  FUNCTION = Radians * 57.29578

END FUNCTION


'------------------------------------------------------------------------------
' GallonsToLiters - Convert gallons (U.S. liquid) to liters
'
FUNCTION GallonsToLiters(BYVAL Gallons) RETPRM EXPORT

  FUNCTION = Gallons * 3.7853

END FUNCTION


'------------------------------------------------------------------------------
' LitersToGallons - Convert liters to gallons (U.S. liquid)
'
FUNCTION LitersToGallons(BYVAL Liters) RETPRM EXPORT

  FUNCTION = Liters * 0.26418

END FUNCTION

'==============================================================================
'                          Transcendental functions
'==============================================================================

'------------------------------------------------------------------------------
' Secant - Secant
'
FUNCTION Sec(BYVAL Value) RETPRM EXPORT

  FUNCTION = 1 / COS(Value)

END FUNCTION


'------------------------------------------------------------------------------
' Cosecant - Cosecant
'
FUNCTION Cosec(BYVAL Value) RETPRM EXPORT

  FUNCTION = 1 / SIN(Value)

END FUNCTION


'------------------------------------------------------------------------------
' Cotangent - Cotangent
'
FUNCTION Cotangent(BYVAL Value) RETPRM EXPORT

  FUNCTION = 1 / TAN(Value)

END FUNCTION


'------------------------------------------------------------------------------
' ArcSine - Inverse Sine
'
FUNCTION  ArcSin(BYVAL Value) RETPRM EXPORT

  FUNCTION = ATN(Value / SQR(-Value * Value + 1))

END FUNCTION


'------------------------------------------------------------------------------
' ArcCos - Inverse Cosine
'
FUNCTION ArcCos(BYVAL Value) RETPRM EXPORT

  FUNCTION = ATN(Value / SQR(-Value * Value + 1)) + 1.570796326794895

END FUNCTION


'------------------------------------------------------------------------------
' ArcSec - Inverse Secant
'
FUNCTION ArcSec(BYVAL Value) RETPRM EXPORT

  FUNCTION = ATN(Value / SQR(-Value * Value - 1)) + SGN(SGN(Value ) - 1) * 1.5708

END FUNCTION


'------------------------------------------------------------------------------
' ArcCosec - Inverse Cosecant
'
FUNCTION ArcCosec(BYVAL Value) RETPRM EXPORT

  FUNCTION = ATN(Value/SQR(Value * Value - 1)) + (SGN(Value)-1) * 1.570796326794895

END FUNCTION


'------------------------------------------------------------------------------
' ArcCotan - Inverse Cotangent
'
FUNCTION ArcCotan(BYVAL Value) RETPRM EXPORT

  FUNCTION = ATN(Value) + 1.570796326794895

END FUNCTION


'------------------------------------------------------------------------------
' SinH - Hyperbolic Sine
'
FUNCTION SinH(BYVAL Value) RETPRM EXPORT

  FUNCTION = (EXP(Value) - EXP(-Value)) / 2

END FUNCTION


'------------------------------------------------------------------------------
' CosH - Hyperbolic Cosine
'
FUNCTION CosH(BYVAL Value) RETPRM EXPORT

  FUNCTION = (EXP(Value) + EXP(-Value)) / 2

END FUNCTION


'------------------------------------------------------------------------------
' TanH - Hyperbolic Tangent
'
FUNCTION TanH(BYVAL Value) RETPRM EXPORT

  FUNCTION = (EXP(Value) - EXP(-Value)) / (EXP(Value) + EXP(-Value))

END FUNCTION


'------------------------------------------------------------------------------
' SecH - Hyperbolic Secant
'
FUNCTION SecH(BYVAL Value) RETPRM EXPORT

  FUNCTION = 2 / (EXP(Value) + EXP(-Value))

END FUNCTION


'------------------------------------------------------------------------------
' CosecH - Hyperbolic Cosecant
'
FUNCTION CosecH(BYVAL Value) RETPRM EXPORT

  FUNCTION = 2 / (EXP(Value) - EXP(-Value))

END FUNCTION


'------------------------------------------------------------------------------
' CotanH - Hyperbolic Cotangent
'
FUNCTION CotanH(BYVAL Value) RETPRM EXPORT

  FUNCTION = (EXP(Value) + EXP(-Value)) / (EXP(Value) + EXP(-Value))

END FUNCTION


'------------------------------------------------------------------------------
' ArcSinH - Inverse Hyperbolic Sine
'
FUNCTION ArcSinH(BYVAL Value) RETPRM EXPORT

  FUNCTION = LOG(Value + SQR(Value * Value + 1))

END FUNCTION


'------------------------------------------------------------------------------
' ArcCosH - Inverse Hyperbolic Cosine
'
FUNCTION ArcCosH(BYVAL Value) RETPRM EXPORT

  FUNCTION = LOG(Value + SQR(Value * Value - 1))

END FUNCTION


'------------------------------------------------------------------------------
' ArcTanH - Inverse Hyperbolic Tangent
'
FUNCTION ArcTanH(BYVAL Value) RETPRM EXPORT

  FUNCTION = LOG((1 + Value) / (1 - Value)) / 2

END FUNCTION


'------------------------------------------------------------------------------
' ArcSecH - Inverse Hyperbolic Secant
'
FUNCTION ArcSecH(BYVAL Value) RETPRM EXPORT

  FUNCTION = LOG((SQR(-Value * Value + 1) + 1) / Value)

END FUNCTION


'------------------------------------------------------------------------------
' ArcCosecH - Inverse Hyperbolic Cosecant
'
FUNCTION ArcCosecH(BYVAL Value) RETPRM EXPORT

  FUNCTION = LOG((SGN(Value) * SQR(Value * Value + 1) + 1) / Value)

END FUNCTION


'------------------------------------------------------------------------------
' ArcCotanH - Inverse Hyperbolic Cotangent
'
FUNCTION ArcCotanH(BYVAL Value) RETPRM EXPORT

  FUNCTION = LOG((Value + 1) / (Value - 1)) / 2

END FUNCTION


'------------------------------------------------------------------------------
' LogN - Logarithm Base N
'
FUNCTION LogN(BYVAL Value, BYVAL N) RETPRM EXPORT

  FUNCTION = LOG(Value) / LOG(N)

END FUNCTION
